@@ -1,17 +1,20 @@
-/*
- * Copyright 2002-2005 The Apache Software Foundation.
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License. You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied. See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 /*
@@ -21,7 +24,7 @@
  *
  * Author(s): Berin Lautenbach
  *
- * $Id: WinCAPICryptoKeyRSA.cpp 1002847 2010-09-29 21:16:44Z scantor $
+ * $Id: WinCAPICryptoKeyRSA.cpp 1138719 2011-06-23 02:49:32Z scantor $
  *
  */
 
@@ -42,6 +45,10 @@ XSEC_USING_XERCES(ArrayJanitor);
 #	define KP_OAEP_PARAMS          36
 #endif
 
+#if !defined (CRYPT_DECRYPT_RSA_NO_PADDING_CHECK)
+#   define CRYPT_DECRYPT_RSA_NO_PADDING_CHECK   0x00000020
+#endif
+
 WinCAPICryptoKeyRSA::WinCAPICryptoKeyRSA(HCRYPTPROV prov) {
 
 	// Create a new key to be loaded as we go
@@ -586,12 +593,12 @@ unsigned int WinCAPICryptoKeyRSA::privateDecrypt(const unsigned char * inBuf,
 		if (!CryptDecrypt(m_key,
 						 0,
 						 TRUE,
-						 CRYPT_OAEP,
+						 (hm == HASH_SHA1) ? CRYPT_OAEP : CRYPT_DECRYPT_RSA_NO_PADDING_CHECK,
 						 plainBuf,
 						 &decryptSize)) {
 
 			throw XSECCryptoException(XSECCryptoException::RSAError,
-				"WinCAPI:RSA privateKeyDecrypt - Error Decrypting PKCS1_5 padded RSA encrypt");
+				"WinCAPI:RSA privateKeyDecrypt - Error Decrypting PKCS1v2 OAEP padded RSA encrypt");
 
 		}
 
@@ -658,6 +665,11 @@ unsigned int WinCAPICryptoKeyRSA::publicEncrypt(const unsigned char * inBuf,
 
 	case XSECCryptoKeyRSA::PAD_OAEP_MGFP1 :
 
+	    if (hm != HASH_SHA1) {
+	        throw XSECCryptoException(XSECCryptoException::RSAError,
+	            "WinCAPI:RSA - OAEP padding method requires SHA-1 digest method");
+	    }
+
 		if (!CryptEncrypt(m_key,
 						  0,			/* No Hash */
 						  TRUE,			/* Is Final */
