@@ -1,17 +1,20 @@
-/*
- * Copyright 2002-2010 The Apache Software Foundation.
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License. You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied. See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 /*
@@ -19,7 +22,7 @@
  *
  * XSECDOMUtils:= Utilities to manipulate DOM within XML-SECURITY
  *
- * $Id: XSECDOMUtils.cpp 958797 2010-06-29 00:44:27Z scantor $
+ * $Id: XSECDOMUtils.cpp 1139478 2011-06-25 01:39:11Z scantor $
  *
  */
 
@@ -696,8 +699,8 @@ XMLCh * decodeDName(const XMLCh * toDecode) {
 static bool isHexDigit(const XMLCh toCheck)
 {
     if ((toCheck >= chDigit_0) && (toCheck <= chDigit_9)
-    ||  (toCheck >= chLatin_A) && (toCheck <= chLatin_Z)
-    ||  (toCheck >= chLatin_a) && (toCheck <= chLatin_z))
+    ||  (toCheck >= chLatin_A) && (toCheck <= chLatin_F)
+    ||  (toCheck >= chLatin_a) && (toCheck <= chLatin_f))
     {
         return true;
     }
@@ -709,46 +712,51 @@ static unsigned int xlatHexDigit(const XMLCh toXlat)
     if ((toXlat >= chDigit_0) && (toXlat <= chDigit_9))
         return (unsigned int)(toXlat - chDigit_0);
 
-    if ((toXlat >= chLatin_A) && (toXlat <= chLatin_Z))
+    if ((toXlat >= chLatin_A) && (toXlat <= chLatin_F))
         return (unsigned int)(toXlat - chLatin_A) + 10;
 
     return (unsigned int)(toXlat - chLatin_a) + 10;
 }
 
-XMLCh * cleanURIEscapes(const XMLCh * str) {
+XMLCh * cleanURIEscapes(const XMLCh * uriPath) {
 
-	// Taken from Xerces XMLURI.cpp
+    XMLByte *ptr, *utf8Path;
+    xsecsize_t len = XMLString::stringLen(uriPath);
 
-	XMLCh * retPath = XMLString::replicate(str);
+    ptr = utf8Path = new XMLByte[len + 1];
 
-	xsecsize_t len = XMLString::stringLen(retPath);
-	int percentIndex = XMLString::indexOf(retPath, chPercent, 0);
+    for (xsecsize_t i = 0; i < len; i++) {
+        unsigned int value = uriPath[i];
 
-	while (percentIndex != -1) {
+        if (value > 255) {
+            delete[] utf8Path;
+            throw XSECException(XSECException::ErrorOpeningURI, "Wide character in URI");
+        }
 
-		if (percentIndex+2 >= len ||
-			!isHexDigit(retPath[percentIndex+1]) ||
-			!isHexDigit(retPath[percentIndex+2]))
-		    XSEC_RELEASE_XMLCH(retPath);
-			throw XSECException(XSECException::ErrorOpeningURI,
-					"Bad escape sequence in URI");
+        if (value == chPercent) {
+            if (!(i + 2 < len && isHexDigit(uriPath[i + 1]) && isHexDigit(uriPath[i + 2]))) {
+                delete[] utf8Path;
+                throw XSECException(XSECException::ErrorOpeningURI, "Bad escape sequence in URI");
+            }
 
-		unsigned int value = (xlatHexDigit(retPath[percentIndex+1]) * 16) +
-						     (xlatHexDigit(retPath[percentIndex+2]));
+            value = (xlatHexDigit(uriPath[i + 1]) * 16) + (xlatHexDigit(uriPath[i + 2]));
+            i += 2;
+        }
 
-		retPath[percentIndex] = XMLCh(value);
-		int i = 0;
-		for (i = percentIndex+1 ; i < len - 2; ++i)
-			retPath[i] = retPath[i+2];
-		retPath[i] = chNull;
-		len = i;
+        *(ptr++) = (XMLByte) value;
+    }
+    *ptr = 0;
 
-		percentIndex = XMLString::indexOf(retPath, chPercent, ++percentIndex);
-
-	}
-
-	return retPath;
+    try {
+        XMLCh* unicodePath = transcodeFromUTF8(utf8Path);
+        delete[] utf8Path;
+        return unicodePath;
+    }
+    catch (XMLException&) {
+    }
 
+    delete[] utf8Path;
+    throw XSECException(XSECException::ErrorOpeningURI, "Failed to transcode URI from UTF-8");
 }
 
 // --------------------------------------------------------------------------------
