@@ -1,17 +1,20 @@
-/*
- * Copyright 2003-2010 The Apache Software Foundation.
+/**
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements. See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership. The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License. You may obtain a copy of the License at
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * http://www.apache.org/licenses/LICENSE-2.0
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied. See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 
 /*
@@ -20,7 +23,7 @@
  * XSECAlgorithmHandlerDefault := Interface class to define handling of
  *								  default encryption algorithms
  *
- * $Id: XENCAlgorithmHandlerDefault.cpp 991761 2010-09-01 23:24:03Z scantor $
+ * $Id: XENCAlgorithmHandlerDefault.cpp 1138719 2011-06-23 02:49:32Z scantor $
  *
  */
 
@@ -585,11 +588,21 @@ unsigned int XENCAlgorithmHandlerDefault::doRSADecryptToSafeBuffer(
 	}
 	else if (strEquals(encryptionMethod->getAlgorithm(), DSIGConstants::s_unicodeStrURIRSA_OAEP_MGFP1)) {
 
+        hashMethod hm;
 	    const XMLCh* digmeth = encryptionMethod->getDigestMethod();
-		if (digmeth && *digmeth && !strEquals(digmeth, DSIGConstants::s_unicodeStrURISHA1)) {
-			throw XSECException(XSECException::CipherError, 
-				"XENCAlgorithmHandlerDefault::doRSADecryptToSafeBuffer - Currently only SHA-1 is supported for OAEP");
-		}
+
+	    // Is this a URI we recognize?
+	    if (!digmeth || !*digmeth) {
+	        hm = HASH_SHA1;
+	    }
+	    else if (!XSECmapURIToHashMethod(digmeth, hm)) {
+	        safeBuffer sb;
+	        sb.sbTranscodeIn("XENCAlgorithmHandlerDefault - Unknown Digest URI : ");
+	        sb.sbXMLChCat(digmeth);
+
+	        throw XSECException(XSECException::AlgorithmMapperError,
+	            sb.rawXMLChBuffer());
+	    }
 
 		// Read out any OAEP params
 		unsigned char * oaepParamsBuf = NULL;
@@ -622,7 +635,7 @@ unsigned int XENCAlgorithmHandlerDefault::doRSADecryptToSafeBuffer(
 												  offset, 
 												  rsa->getLength(), 
 												  XSECCryptoKeyRSA::PAD_OAEP_MGFP1, 
-												  HASH_SHA1);
+												  hm);
 
 	}
 
